"""Utility Base Classes.

This module contains utility base classes that are used throughout the
application.

"""


from .saved_class import SavedClass
from .settings import Settings
from .icons import Icons
from .exceptions import *
